<?php

    namespace App\Jobs;

    use App\Exceptions\CampaignPausedException;
    use App\Library\Exception\NoCreditsLeft;
    use App\Library\Exception\QuotaExceeded;
    use DateTime;
    use Exception;
    use Illuminate\Bus\Batchable;
    use Illuminate\Bus\Queueable;
    use Illuminate\Contracts\Queue\ShouldQueue;
    use Illuminate\Foundation\Bus\Dispatchable;
    use Illuminate\Queue\InteractsWithQueue;
    use Illuminate\Queue\SerializesModels;
    use Illuminate\Support\Facades\Log as LaravelLog;
    use Throwable;
    use function App\Helpers\plogger;

    class SendFileMessage implements ShouldQueue
    {

        use Batchable;
        use Dispatchable;
        use InteractsWithQueue;
        use Queueable;
        use SerializesModels;

        public int $timeout = 600;

        protected $sendData;
        protected $campaign;
        protected $triggerId;
        protected $server;
        protected $stopOnError = false;

        /**
         * Create a new job instance.
         */
        public function __construct($campaign, $sendData, $triggerId = null)
        {
            $this->campaign  = $campaign;
            $this->sendData  = $sendData;
            $this->triggerId = $triggerId;
            $this->server    = $sendData->sendingServer;
        }

        /**
         * @throws Exception
         */
        public function setStopOnError($value): void
        {
            if ( ! is_bool($value)) {
                throw new Exception('Parameter passed to setStopOnError must be bool');
            }

            $this->stopOnError = $value;
        }


        /**
         * Determine the time at which the job should timeout.
         *
         * @return DateTime
         */
        public function retryUntil(): DateTime
        {
            return now()->addHours(12);
        }


        /**
         * @throws QuotaExceeded
         * @throws Exception
         */
        public function handle(): void
        {
            if ($this->batch() && $this->batch()->cancelled()) {
                return;
            }
            $this->send();

        }

        /**
         * @throws Exception
         */
        public function send($exceptionCallback = null)
        {

            $logger  = $this->campaign->logger();
            $plogger = plogger($this->campaign->uid);
            $phone   = $this->sendData->phone;

            $logger->info(sprintf('Sending to %s [Server "%s"]', $phone, $this->server->name));
            $plogger->info(sprintf('Sending to %s [Server "%s"]', $phone, $this->server->name));


            try {

                if ($this->campaign->user->sms_unit != '-1' && $this->campaign->user->sms_unit == 0) {
                    throw new CampaignPausedException(sprintf("Campaign `%s` (%s) halted, customer exceeds sms units limit", $this->campaign->campaign_name, $this->campaign->uid));
                }


                $sms_type = $this->sendData->sms_type;

                $preparedData = [
                    'user_id'        => $this->campaign->user_id,
                    'phone'          => $this->sendData->phone,
                    'sender_id'      => $this->sendData->sender_id,
                    'message'        => $this->sendData->message,
                    'sms_type'       => $sms_type,
                    'cost'           => $this->sendData->cost,
                    'sms_count'      => $this->sendData->sms_count,
                    'campaign_id'    => $this->campaign->id,
                    'sending_server' => $this->sendData->sendingServer,
                ];


                if ($sms_type == 'voice') {
                    $preparedData['language'] = $this->campaign->language;
                    $preparedData['gender']   = $this->campaign->gender;
                }

                if ($sms_type == 'mms' || $sms_type == 'whatsapp' || $sms_type == 'viber') {
                    if (isset($this->campaign->media_url)) {
                        $preparedData['media_url'] = $this->campaign->media_url;
                    }

                    if (isset($this->campaign->language)) {
                        $preparedData['language'] = $this->campaign->language;
                    }
                }

                $getData = $this->campaign->sendSMS($preparedData);
                $this->campaign->updateCache(substr_count($getData->status, 'Delivered') == 1 ? 'DeliveredCount' : 'FailedDeliveredCount');
                $this->sendData->delete();

                if (substr_count($getData->status, 'Delivered') == 1 && $this->campaign->user->sms_unit != '-1') {
                    $this->campaign->user->countSMSUnit($getData->cost);
                }

            } catch (Throwable|CampaignPausedException $ex) {
                if ( ! is_null($exceptionCallback)) {
                    return $exceptionCallback($ex);
                }

                $message = sprintf("Error sending to [%s]. Error: %s", $phone, $ex->getMessage());
                LaravelLog::error('ERROR SENDING EMAIL (debug): ' . $ex->getTraceAsString());
                $logger->error($message);
                $plogger->error($message);

                // In case of these exceptions, stop campaign immediately even if stopOnError is currently false
                // This is helpful in certain cases: for example, when credits runs out, then it does not make sense to keep sending (and failing)
                $forceEndCampaignExceptions = [
                    NoCreditsLeft::class,
                    // Other "end-game" exception like "SendingServer out of credits, etc."
                ];

                $forceEndCampaign = in_array(get_class($ex), $forceEndCampaignExceptions);

                // There are 2 options here
                // Option 1: throw an exception and show it to users as the campaign status
                //     throw new Exception($message);
                // Option 2: just skip the error, log it and proceed with the next subscriber
                if ($this->stopOnError || $forceEndCampaign) {
                    $this->campaign->pause($message);
                    $this->batch()->cancel();
                }
            } finally {
                //
            }

            $plogger->info('SendMessage: ALL done');

            return true;
        }

    }
